@@ -34,7 +34,9 @@
 // If you do not wish that, delete this exception notice.
 //
 
-#include <config.h>
+#ifdef  NEW_STDCPP
+
+#include <ucommon-config.h>
 #include <commoncpp/config.h>
 #include <commoncpp/thread.h>
 #include <commoncpp/slog.h>
@@ -1187,4 +1189,4 @@ AppLog& AppLog::operator<< (ostream& (*pfManipulator)(ostream&))
   return  *this ;
 }
 
-
+#endif
