@@ -36,7 +36,7 @@
 // If you do not wish that, delete this exception notice.
 //
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <commoncpp/config.h>
 #include <commoncpp/export.h>
 #include <commoncpp/string.h>
@@ -197,8 +197,8 @@ Socket(fam, SOCK_DCCP, IPPROTO_DCCP)
 #ifdef  CCXX_IPV6
     struct sockaddr_in6 addr6;
 #endif
-    struct sockaddr_in *ap;
-  socklen_t alen = 0;
+    struct sockaddr *ap;
+    socklen_t alen = 0;
 
     struct servent *svc;
 
@@ -243,19 +243,21 @@ Socket(fam, SOCK_DCCP, IPPROTO_DCCP)
 
     switch(family) {
 #ifdef  CCXX_IPV6
-    case IPV6:
+    case IPV6: {
         IPV6Address ia6(name);
         addr6.sin6_port = addr.sin_port;
         addr6.sin6_family = family;
-        ap = &addr6;
+        addr6.sin6_addr = getaddress(ia6);
+        ap = (struct sockaddr *)&addr6;
         alen = sizeof(addr6);
         break;
+    }
 #endif
     case IPV4:
         IPV4Address ia(name);
         addr.sin_addr = getaddress(ia);
-        ap = &addr;
-    alen = sizeof(addr);
+        ap = (struct sockaddr *)&addr;
+        alen = sizeof(addr);
     }
 
     if(bind(so, (struct sockaddr *)ap, alen)) {
@@ -436,7 +438,6 @@ void DCCPSocket::connect(const char *target)
 {
     char namebuf[128];
     char *cp;
-    bool connected = false;
     struct servent *svc;
     tpport_t port;
 
