@@ -36,6 +36,10 @@
 // If you do not wish that, delete this exception notice.
 //
 
+#ifndef _MSC_VER
+#include <sys/stat.h>
+#endif
+
 // needed for GNU/LINUX glibc otherwise pread/pwrite wont work
 
 #ifndef _XOPEN_SOURCE
@@ -50,7 +54,16 @@
 #define _XOPEN_SOURCE_EXTENDED
 #endif
 
-#include <config.h>
+#include <ucommon-config.h>
+
+// broken BSD; XOPEN should not imply _POSIX_C_SOURCE,
+//  _POSIX_C_SOURCE should not stop __BSD_VISIBLE
+
+#define u_int unsigned int
+#define u_short unsigned short
+#define u_long unsigned long
+#define u_char unsigned char
+
 #include <fcntl.h>
 
 #ifdef  HAVE_SYS_PARAM_H
@@ -79,6 +92,7 @@
 #include <cstdio>
 #include <cstdlib>
 #endif
+
 #include <sys/stat.h>
 #include <cerrno>
 
@@ -1448,7 +1462,9 @@ char *File::getRealpath(const char *path, char *buffer, size_t size)
     if(size > PATH_MAX)
         size = PATH_MAX;
 
+#ifdef  HAVE_LSTAT
     unsigned symlinks = 0;
+#endif
 #if !defined(DYNAMCIC_LOCAL_ARRAYS)
     char left[PATH_MAX];
 #else
