@@ -36,7 +36,7 @@
 // If you do not wish that, delete this exception notice.
 //
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <commoncpp/config.h>
 #include <commoncpp/export.h>
 #include <commoncpp/thread.h>
@@ -84,9 +84,6 @@
 
 using namespace COMMONCPP_NAMESPACE;
 
-static char *_pUser = NULL;
-static char *_pHome = NULL;
-
 bool Process::rtflag = false;
 
 #ifdef  _MSWINDOWS_
@@ -233,6 +230,9 @@ bool Process::isScheduler(void)
 #define WCOREDUMP(status) (((status) & 0x80) != 0)
 #endif
 
+static char *_pUser = NULL;
+static char *_pHome = NULL;
+
 static void lookup(void)
 {
     struct passwd *pw = NULL;
