@@ -36,7 +36,7 @@
 // If you do not wish that, delete this exception notice.
 //
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <commoncpp/config.h>
 #include <commoncpp/export.h>
 #include <commoncpp/string.h>
@@ -425,7 +425,7 @@ void TimerPort::incTimer(timeout_t timeout)
     int usecs = (timeout % 1000) * 1000;
 
     timer.tv_usec += usecs;
-    if(timer.tv_usec > 1000000l) {
+    if(timer.tv_usec >= 1000000l) {
         ++timer.tv_sec;
         timer.tv_usec %= 1000000l;
     }
