@@ -15,8 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
+#include <ucommon/protocols.h>
+#include <ucommon/object.h>
 #include <ucommon/memory.h>
 #include <ucommon/thread.h>
 #include <ucommon/containers.h>
@@ -485,17 +487,14 @@ bool queue::post(ObjectProtocol *object, timeout_t timeout)
     if(freelist) {
         mem = freelist;
         freelist = freelist->getNext();
-        unlock();
         new((caddr_t)mem) member(this, object);
     }
     else {
-        unlock();
         if(pager)
             new((caddr_t)(pager->alloc(sizeof(member)))) member(this, object);
         else
             new member(this, object);
     }
-    lock();
     signal();
     unlock();
     return true;
@@ -665,11 +664,9 @@ bool stack::push(ObjectProtocol *object, timeout_t timeout)
     if(freelist) {
         mem = freelist;
         freelist = freelist->getNext();
-        unlock();
         new((caddr_t)mem) member(this, object);
     }
     else {
-        unlock();
         if(pager) {
             caddr_t ptr = (caddr_t)pager->alloc(sizeof(member));
             new(ptr) member(this, object);
@@ -677,7 +674,6 @@ bool stack::push(ObjectProtocol *object, timeout_t timeout)
         else
             new member(this, object);
     }
-    lock();
     signal();
     unlock();
     return true;
