@@ -15,11 +15,24 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
+#ifndef _MSC_VER
+#include <sys/stat.h>
+#endif
+
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 600
 #endif
 
-#include <config.h>
+#include <ucommon-config.h>
+
+// broken BSD; XOPEN should not imply _POSIX_C_SOURCE,
+//  _POSIX_C_SOURCE should not stop __BSD_VISIBLE
+
+#define u_int unsigned int
+#define u_short unsigned short
+#define u_long unsigned long
+#define u_char unsigned char
+
 #include <ucommon/export.h>
 #include <ucommon/thread.h>
 #include <ucommon/fsys.h>
@@ -71,7 +84,7 @@
 
 using namespace UCOMMON_NAMESPACE;
 
-const fsys::offset_t fsys::end = (size_t)(-1);
+const fsys::offset_t fsys::end = (offset_t)(-1);
 charfile cstdin(stdin);
 charfile cstdout(stdout);
 charfile cstderr(stderr);
@@ -176,6 +189,32 @@ int fsys::removeDir(const char *path)
     return 0;
 }
 
+fd_t fsys::nullfile(void)
+{
+    SECURITY_ATTRIBUTES sattr;
+
+    sattr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sattr.bInheritHandle = TRUE;
+    sattr.lpSecurityDescriptor = NULL;
+
+    return CreateFile("nul", GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, &sattr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+}
+
+int fsys::pipe(fd_t& input, fd_t& output, size_t size)
+{
+    input = output = NULL;
+    SECURITY_ATTRIBUTES sattr;
+
+    sattr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sattr.bInheritHandle = TRUE;
+    sattr.lpSecurityDescriptor = NULL;
+
+    if(!CreatePipe(&input, &output, &sattr, size))
+        return remapError();
+
+    return 0;
+}
+
 int fsys::fileinfo(const char *path, struct stat *buf)
 {
     if(_stat(path, (struct _stat *)(buf)))
@@ -196,7 +235,7 @@ int fsys::trunc(offset_t offset)
 
 int fsys::fileinfo(struct stat *buf)
 {
-    int fn = _open_osfhandle((long int)(fd), O_RDONLY);
+    int fn = _open_osfhandle((intptr_t)(fd), O_RDONLY);
 
     int rtn = _fstat(fn, (struct _stat *)(buf));
     _close(fn);
@@ -318,17 +357,35 @@ int fsys::sync(void)
 
 fd_t fsys::input(const char *path)
 {
-    return CreateFile(path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+    SECURITY_ATTRIBUTES sattr;
+
+    sattr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sattr.bInheritHandle = TRUE;
+    sattr.lpSecurityDescriptor = NULL;
+
+    return CreateFile(path, GENERIC_READ, FILE_SHARE_READ, &sattr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 }
 
 fd_t fsys::output(const char *path)
 {
-    return CreateFile(path, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+    SECURITY_ATTRIBUTES sattr;
+
+    sattr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sattr.bInheritHandle = TRUE;
+    sattr.lpSecurityDescriptor = NULL;
+
+    return CreateFile(path, GENERIC_WRITE, 0, &sattr, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
 }
 
 fd_t fsys::append(const char *path)
 {
-    fd_t fd = CreateFile(path, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+    SECURITY_ATTRIBUTES sattr;
+
+    sattr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sattr.bInheritHandle = TRUE;
+    sattr.lpSecurityDescriptor = NULL;
+
+    fd_t fd = CreateFile(path, GENERIC_WRITE, 0, &sattr, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
 
     if(fd != INVALID_HANDLE_VALUE)
         SetFilePointer(fd, 0, NULL, FILE_END);
@@ -484,6 +541,26 @@ fsys::fsys(const fsys& copy)
     }
 }
 
+int fsys::inherit(fd_t& from, bool enable)
+{
+    HANDLE pHandle = GetCurrentProcess();
+
+    if(!enable) {
+        if(!SetHandleInformation(from, HANDLE_FLAG_INHERIT, 0))
+            return remapError();
+        return 0;
+    }
+
+    fd_t fd;
+    if(DuplicateHandle(pHandle, from, pHandle, &fd, 0, TRUE, DUPLICATE_SAME_ACCESS)) {
+        release(from);
+        from = fd;
+        return 0;
+    }
+
+    return remapError();
+}
+
 void fsys::operator=(fd_t from)
 {
     HANDLE pHandle = GetCurrentProcess();
@@ -531,7 +608,7 @@ int fsys::seek(offset_t pos)
     DWORD rpos = pos;
     int mode = FILE_BEGIN;
 
-    if(rpos == end) {
+    if(rpos == (DWORD)end) {
         rpos = 0;
         mode = FILE_END;
     }
@@ -600,6 +677,22 @@ ssize_t fsys::write(const void *buf, size_t len)
     return rtn;
 }
 
+fd_t fsys::nullfile(void)
+{
+    return ::open("/dev/null", O_RDWR);
+}
+
+int fsys::pipe(fd_t& input, fd_t& output, size_t size)
+{
+    input = output = -1;
+    int pfd[2];
+    if(::pipe(pfd))
+        return remapError();
+    input = pfd[0];
+    output = pfd[1];
+    return 0;
+}
+
 bool fsys::istty(fd_t fd)
 {
     if(isatty(fd))
@@ -828,6 +921,21 @@ fsys::fsys(const fsys& copy)
     ptr = NULL;
 }
 
+int fsys::inherit(fd_t& fd, bool enable)
+{
+    unsigned long flags;
+    if(fd > -1) {
+        flags = fcntl(fd, F_GETFL);
+        if(enable)
+            flags &= ~FD_CLOEXEC;
+        else
+            flags |= FD_CLOEXEC;
+        if(fcntl(fd, F_SETFL, flags))
+            return remapError();
+    }
+    return 0;
+}
+
 void fsys::operator=(fd_t from)
 {
     close();
@@ -909,6 +1017,14 @@ fsys::~fsys()
     close();
 }
 
+void fsys::operator*=(fd_t& from)
+{
+    if(fd != INVALID_HANDLE_VALUE)
+        close();
+    fd = from;
+    from = INVALID_HANDLE_VALUE;
+}
+
 int fsys::linkinfo(const char *path, char *buffer, size_t size)
 {
 #if defined(_MSWINDOWS_)
@@ -973,7 +1089,7 @@ int fsys::link(const char *path, const char *target)
     WORD len;
 
     lstrcpy(dest, "\\??\\");
-    if(!GetFullPathName(path, sizeof(dest) - (4 * sizeof(TCHAR)), &dest[4], &part) || GetFileAttributes(&dest[4]) == -1)
+    if(!GetFullPathName(path, sizeof(dest) - (4 * sizeof(TCHAR)), &dest[4], &part) || GetFileAttributes(&dest[4]) == INVALID_FILE_ATTRIBUTES)
         return remapError();
 
     memset(reparse, 0, sizeof(reparse));
@@ -1340,9 +1456,10 @@ void fsys::unload(fsys& module)
 
 #else
 
-mem_t fsys::load(const char *path)
+void fsys::load(fsys& module, const char *path)
 {
-    return NULL;
+    module.error = ENOEXEC;
+    module.ptr = NULL;
 }
 
 void fsys::unload(mem_t addr)
@@ -1563,3 +1680,9 @@ int fsys::exec(const char *path, char **argv)
     return shell::wait(pid);
 }
 
+int fsys::exec(const char *path, char **argv, char **envp)
+{
+    shell::pid_t pid = shell::spawn(path, argv, envp);
+    return shell::wait(pid);
+}
+
