@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
 #include <ucommon/linked.h>
 #include <ucommon/string.h>
@@ -399,7 +399,11 @@ int NamedObject::compare(const char *cid) const
 {
     assert(cid != NULL && *cid != 0);
 
+#ifdef  HAVE_STRCOLL
+    return strcoll(id, cid);
+#else
     return strcmp(id, cid);
+#endif
 }
 
 extern "C" {
