@@ -15,8 +15,9 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
+#include <ucommon/object.h>
 #include <ucommon/memory.h>
 #include <ucommon/thread.h>
 #include <ucommon/string.h>
@@ -227,6 +228,169 @@ void *mempager::_alloc(size_t size)
     return mem;
 }
 
+objectpager::member::member(LinkedObject **root) :
+LinkedObject(root)
+{
+    mem = NULL;
+}
+
+objectpager::member::member() :
+LinkedObject()
+{
+    mem = NULL;
+}
+
+objectpager::objectpager(size_t objsize, size_t size) :
+memalloc(size)
+{
+    members = 0;
+    root = NULL;
+    last = NULL;
+    index = NULL;
+    typesize = objsize;
+}
+
+void *objectpager::get(unsigned index)
+{
+    _lock();
+    linked_pointer<member> list = root;
+
+    if(index >= members) {
+        _unlock();
+        return NULL;
+    }
+
+    while(index--)
+        list.next();
+
+    _unlock();
+    return list->mem;
+}
+
+void objectpager::clear(void)
+{
+    memalloc::purge();
+    members = 0;
+    root = NULL;
+    last = NULL;
+    index = NULL;
+}
+
+void *objectpager::pull(void)
+{
+    _lock();
+    if(!members) {
+        _unlock();
+        return NULL;
+    }
+
+    member *mem = (member *)root;
+    void *result = mem->mem;
+    --members;
+    if(!members) {
+        root = NULL;
+        last = NULL;
+    }
+    else
+        root = mem->next;
+    index = NULL;
+    _unlock();
+    return result;
+}
+
+void *objectpager::push(void)
+{
+    _lock();
+    caddr_t mem = (caddr_t)memalloc::_alloc(sizeof(member));
+
+    member *node;
+
+    node = new(mem) member(&root);
+    if(!last)
+        last = node;
+    ++members;
+    node->mem = memalloc::_alloc(typesize);
+    index = NULL;
+    _unlock();
+    return node->mem;
+}
+
+void *objectpager::pop(void)
+{
+    void *out = NULL;
+
+    _lock();
+    if(!root) {
+        _unlock();
+        return NULL;
+    }
+
+    index = NULL;
+
+    if(root == last) {
+        out = last->mem;
+        root = last = NULL;
+        members = 0;
+        _unlock();
+        return out;
+    }
+
+    linked_pointer<member> np = root;
+    while(is(np)) {
+        if(np->next == last) {
+            out = last->mem;
+            last = *np;
+            np->next = NULL;
+            --members;
+            break;
+        }
+        np.next();
+    }
+    _unlock();
+    return out;
+}
+
+void *objectpager::add(void)
+{
+    _lock();
+    caddr_t mem = (caddr_t)memalloc::_alloc(sizeof(member));
+    member *node;
+
+    index = NULL;
+    if(members++) {
+        node = new(mem) member();
+        last->set(node);
+    }
+    else
+        node = new(mem) member(&root);
+    last = node;
+    node->mem = memalloc::_alloc(typesize);
+    _unlock();
+    return node->mem;
+}
+
+void **objectpager::list(void)
+{
+    _lock();
+    void **dp = index;
+    if(dp) {
+        _unlock();
+        return dp;
+    }
+
+    unsigned pos = 0;
+    index = (void **)memalloc::_alloc(sizeof(void *) * (members + 1));
+    linked_pointer<member> mp = root;
+    while(is(mp)) {
+        index[pos++] = mp->mem;
+        mp.next();
+    }
+    index[pos] = NULL;
+    dp = index;
+    _unlock();
+    return index;
+}
+
 stringpager::member::member(LinkedObject **root, const char *data) :
 LinkedObject(root)
 {
@@ -274,6 +438,90 @@ unsigned stringpager::token(const char *text, const char *list, const char *quot
     return count;
 }
 
+String stringpager::join(const char *prefix, const char *middle, const char *suffix)
+{
+    string_t tmp;
+
+    if(!members)
+        return tmp;
+
+    if(prefix && *prefix)
+        tmp += prefix;
+
+    linked_pointer<member> mp = root;
+    while(is(mp)) {
+        tmp += mp->text;
+        if(mp->next && middle && *middle)
+            tmp += middle;
+        else if(mp->next == NULL && suffix && *suffix)
+            tmp += suffix;
+        mp.next();
+    }
+
+    return tmp;
+}
+
+unsigned stringpager::split(const char *text, const char *string, unsigned flags)
+{
+    strdup_t tmp = String::dup(string);
+    char *match = tmp;
+    char *prior = tmp;
+    size_t tcl = strlen(text);
+    unsigned count = 0;
+    bool found = false;
+
+    // until end of string or end of matches...
+    while(prior && *prior && match) {
+#if defined(_MSWINDOWS_)
+        if((flags & 0x01) == String::INSENSITIVE)
+            match = strstr(prior, text);
+#elif  defined(HAVE_STRICMP)
+        if((flags & 0x01) == String::INSENSITIVE)
+            match = stristr(prior, text);
+#else
+        if((flags & 0x01) == String::INSENSITIVE)
+            match = strcasestr(prior, text);
+#endif
+        else
+            match = strstr(prior, text);
+
+        if(match)
+            found = true;
+
+        // we must have at least one match to add trailing data...
+        if(match == NULL && prior != NULL && found) {
+            ++count;
+            add(prior);
+        }
+        // if we have a current match see if anything to add in front of it
+        else if(match) {
+            *match = 0;
+            if(match > prior) {
+                ++count;
+                add(prior);
+            }
+
+            prior = match + tcl;
+        }
+    }
+    return count;
+}
+
+void stringpager::set(unsigned index, const char *text)
+{
+    linked_pointer<member> list = root;
+
+    if(index >= members)
+
+    while(index--)
+        list.next();
+
+    size_t size = strlen(text) + 1;
+    char *str = (char *)memalloc::_alloc(size);
+    strcpy(str, text);
+    list->text = str;
+}
+
 const char *stringpager::get(unsigned index)
 {
     linked_pointer<member> list = root;
@@ -675,6 +923,45 @@ void *keyassoc::remove(const char *id)
     return data;
 }
 
+void *keyassoc::allocate(char *id, size_t dsize)
+{
+    assert(id != NULL && *id != 0);
+    assert(dsize != 0);
+
+    void *dp = NULL;
+    keydata *kd;
+    LinkedObject *obj;
+    unsigned size = strlen(id);
+
+    if(keysize && size >= keysize)
+        return NULL;
+
+    _lock();
+    kd = static_cast<keydata *>(NamedObject::map(root, id, paths));
+    if(kd) {
+        _unlock();
+        return NULL;
+    }
+    caddr_t ptr = NULL;
+    size /= 8;
+    if(list && list[size]) {
+        obj = list[size];
+        list[size] = obj->getNext();
+        ptr = (caddr_t)obj;
+    }
+    if(ptr == NULL) {
+        ptr = (caddr_t)memalloc::_alloc(sizeof(keydata) + size * 8);
+        dp = memalloc::_alloc(dsize);
+    }
+    else
+        dp = ((keydata *)(ptr))->data;
+    kd = new(ptr) keydata(this, id, paths, 8 + size * 8);
+    kd->data = dp;
+    ++count;
+    _unlock();
+    return dp;
+}
+
 bool keyassoc::create(char *id, void *data)
 {
     assert(id != NULL && *id != 0);
