@@ -15,10 +15,13 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
+#include <ucommon/protocols.h>
 #include <ucommon/string.h>
 #include <ucommon/memory.h>
+#include <ucommon/thread.h>
+#include <ucommon/fsys.h>
 #include <ucommon/shell.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -99,312 +102,6 @@ static shell::exitproc_t _exitproc = NULL;
 static shell::numeric_t numeric_mode = shell::NO_NUMERIC;
 static long numeric_value = 0l;
 
-shell::pipeio::pipeio()
-{
-    pid = INVALID_PID_VALUE;
-    input = output = INVALID_HANDLE_VALUE;
-    perror = presult = 0;
-}
-
-#ifdef  _MSWINDOWS_
-
-int shell::pipeio::spawn(const char *path, char **argv, pmode_t mode, size_t size, char **envp)
-{
-    SECURITY_ATTRIBUTES sa;
-    fd_t stdio[3] = {INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE};
-
-    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
-    sa.lpSecurityDescriptor = NULL;
-    sa.bInheritHandle = TRUE;
-
-    if(mode == RD || mode == RDWR)
-        CreatePipe(&input, &stdio[1], &sa, size);
-
-    if(mode == WR || mode == RDWR)
-        CreatePipe(&stdio[0], &output, &sa, size);
-
-    pid = shell::spawn(path, argv, envp, stdio);
-    perror = 0;
-    if(pid == INVALID_PID_VALUE) {
-        perror = fsys::remapError();
-        if(mode == RD || mode == RDWR) {
-            CloseHandle(input);
-            input = INVALID_HANDLE_VALUE;
-        }
-        if(mode == WR || mode == RDWR) {
-            CloseHandle(output);
-            output = INVALID_HANDLE_VALUE;
-        }
-    }
-
-    if(mode == RD || mode == RDWR)
-        CloseHandle(stdio[1]);
-    if(mode == WR || mode == RDWR)
-        CloseHandle(stdio[0]);
-
-    return perror;
-}
-
-
-int shell::pipeio::cancel(void)
-{
-    if(pid != INVALID_PID_VALUE)
-        TerminateProcess(pid, 7);
-
-    return wait();
-}
-
-int shell::pipeio::wait(void)
-{
-    DWORD code;
-
-    presult = -1;
-
-    // if stdio, we do not want to close stdin/stdout handles
-    // instead we just mark the object to disable buffering
-    if(pid == INVALID_PID_VALUE) {
-        perror = EPIPE;
-        return presult;
-    }
-
-    if(input != INVALID_HANDLE_VALUE)
-        CloseHandle(input);
-
-    if(output != INVALID_HANDLE_VALUE)
-        CloseHandle(output);
-
-    input = output = INVALID_HANDLE_VALUE;
-
-    if(WaitForSingleObject(pid, INFINITE) == WAIT_FAILED) {
-        pid = INVALID_PID_VALUE;
-        return -1;
-    }
-
-    GetExitCodeProcess(pid, &code);
-    CloseHandle(pid);
-    pid = INVALID_PID_VALUE;
-    presult = code;
-    return (int)code;
-}
-
-size_t shell::pipeio::read(void *address, size_t size)
-{
-    DWORD count;
-
-    if(input == INVALID_HANDLE_VALUE || perror)
-        return 0;
-
-    if(ReadFile(input, (LPVOID)address, (DWORD)size, &count, NULL))
-        return (size_t)count;
-
-    perror = fsys::remapError();
-    return 0;
-}
-
-size_t shell::pipeio::write(const void *address, size_t size)
-{
-    DWORD count;
-
-    if(output == INVALID_HANDLE_VALUE || perror)
-        return 0;
-
-    if(WriteFile(output, (LPVOID)address, (DWORD)size, &count, NULL))
-        return (size_t)count;
-
-    perror = fsys::remapError();
-    return 0;
-}
-
-#else
-
-int shell::pipeio::spawn(const char *path, char **argv, pmode_t mode, size_t buffer, char **envp)
-{
-    fd_t pin[2], pout[2];
-    fd_t stdio[3] = {INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE, INVALID_HANDLE_VALUE};
-
-    pin[0] = INVALID_HANDLE_VALUE;
-    pout[1] = INVALID_HANDLE_VALUE;
-    stdio[2] = 2;
-
-    if(mode == RD || mode == RDWR) {
-        if(::pipe(pin)) {
-            perror = errno;
-            return perror;
-        }
-        stdio[1] = pin[1];
-    }
-
-    if(mode == WR || mode == RDWR) {
-        if(::pipe(pout)) {
-            perror = errno;
-            if(mode == RDWR) {
-                ::close(pin[0]);
-                ::close(pin[1]);
-            }
-            return perror;
-        }
-        stdio[0] = pout[0];
-    }
-
-    pid = shell::spawn(path, argv, envp, stdio);
-
-    if(mode == RD || mode == RDWR)
-        ::close(pin[1]);
-    if(mode == WR || mode == RDWR)
-        ::close(pout[0]);
-
-    perror = 0;
-    if(pid == INVALID_PID_VALUE) {
-        perror = errno;
-        if(mode == RD || mode == RDWR) {
-            ::close(pin[0]);
-            pin[0] = INVALID_HANDLE_VALUE;
-        }
-        if(mode == WR || mode == RDWR) {
-            ::close(pout[1]);
-            pout[1] = INVALID_HANDLE_VALUE;
-        }
-    }
-
-    input = pin[0];
-    output = pout[1];
-    return perror;
-}
-
-int shell::pipeio::wait(void)
-{
-    presult = -1;
-
-    // we do not want to close our stdin/stdout handles
-    if(pid == INVALID_PID_VALUE) {
-        perror = EPIPE;
-        return -1;
-    }
-
-    if(input != INVALID_HANDLE_VALUE)
-        ::close(input);
-
-    if(output != INVALID_HANDLE_VALUE)
-        ::close(output);
-
-    input = output = INVALID_HANDLE_VALUE;
-
-    if(::waitpid(pid, &presult, 0) != pid)
-        presult = -1;
-
-    pid = INVALID_PID_VALUE;
-
-    if(presult == -1)
-        return -1;
-
-    presult = WEXITSTATUS(presult);
-    return presult;
-}
-
-int shell::pipeio::cancel(void)
-{
-    if(pid != INVALID_PID_VALUE)
-        kill(pid, SIGTERM);
-
-    return wait();
-}
-
-size_t shell::pipeio::read(void *address, size_t size)
-{
-    if(input == INVALID_HANDLE_VALUE || perror)
-        return 0;
-
-    ssize_t result = ::read(input, address, size);
-    if(result < 0) {
-        perror = fsys::remapError();
-        return 0;
-    }
-    return (size_t)result;
-}
-
-size_t shell::pipeio::write(const void *address, size_t size)
-{
-    if(output == INVALID_HANDLE_VALUE || perror)
-        return 0;
-
-    ssize_t result = ::write(output, address, size);
-    if(result < 0) {
-        perror = fsys::remapError();
-        return 0;
-    }
-    return (size_t)result;
-}
-
-#endif
-
-shell::iobuf::iobuf(const char *path, char **argv, shell::pmode_t mode, size_t size, char **env) :
-BufferProtocol(), shell::pipeio()
-{
-    ioerror = 0;
-    open(path, argv, mode, size, env);
-}
-
-shell::iobuf::iobuf(size_t size) :
-BufferProtocol(), shell::pipeio()
-{
-    ioerror = 0;
-
-    if(size) {
-        pipeio::input = shell::input();
-        pipeio::output = shell::output();
-        allocate(size, BUF_RDWR);
-    }
-}
-
-shell::iobuf::~iobuf()
-{
-    cancel();
-}
-
-void shell::iobuf::_clear(void)
-{
-    ioerror = 0;
-}
-
-int shell::iobuf::_err(void) const
-{
-    return ioerror;
-}
-
-size_t shell::iobuf::_pull(char *buf, size_t size)
-{
-    size_t result = pipeio::read(buf, size);
-    if(perror)
-        ioerror = perror;
-    return result;
-}
-
-size_t shell::iobuf::_push(const char *buf, size_t size)
-{
-    size_t result = pipeio::write(buf, size);
-    if(perror)
-        ioerror = perror;
-    return result;
-}
-
-void shell::iobuf::close(void)
-{
-    pipeio::wait();
-    BufferProtocol::release();
-}
-
-void shell::iobuf::cancel(void)
-{
-    pipeio::cancel();
-    BufferProtocol::release();
-}
-
-void shell::iobuf::open(const char *path, char **argv, shell::pmode_t mode, size_t size, char **env)
-{
-    if(!pipeio::spawn(path, argv, mode, size, env))
-        BufferProtocol::allocate(size, (type_t)mode);
-}
-
 shell::Option::Option(char shortopt, const char *longopt, const char *value, const char *help) :
 OrderedObject()
 {
@@ -1168,38 +865,6 @@ void shell::errexit(int exitcode, const char *format, ...)
     ::exit(exitcode);
 }
 
-size_t shell::readln(pipe_t pipe, char *buffer, size_t size)
-{
-    unsigned pos = 0;
-
-    while(pos < size - 1) {
-        if(pipe->read(buffer, 1) < 1)
-            break;
-        if(buffer[pos] == '\n')
-            break;
-        ++pos;
-    }
-
-    if(pos && buffer[pos - 1] == '\r')
-        --pos;
-
-    buffer[pos] = 0;
-    return pos;
-}
-
-size_t shell::printf(pipe_t pipe, const char *format, ...)
-{
-    char buf[512];
-
-    va_list args;
-    va_start(args, format);
-    vsnprintf(buf, sizeof(buf), format, args);
-    va_end(args);
-    if(buf[0])
-        return pipe->write(buf, strlen(buf));
-    return 0;
-}
-
 size_t shell::printf(const char *format, ...)
 {
     va_list args;
@@ -1237,19 +902,6 @@ size_t shell::read(String& string)
     return size;
 }
 
-size_t shell::writes(pipe_t pipe, const char *string)
-{
-    size_t size = 0;
-
-    if(string)
-        size = strlen(string);
-
-    if(size)
-        return pipe->write(string, size);
-
-    return 0;
-}
-
 size_t shell::writes(const char *string)
 {
     size_t result = fputs(string, stdout);
@@ -1258,44 +910,6 @@ size_t shell::writes(const char *string)
     return result;
 }
 
-size_t shell::read(pipe_t pipe, String& str)
-{
-    char *cp = str.c_mem();
-    size_t size = str.size();
-
-    if(!size)
-        return 0;
-
-    size = readln(pipe, cp, size);
-    cp[size] = 0;
-    String::fix(str);
-    return size;
-}
-
-int shell::cancel(shell::pipe_t pio)
-{
-    int status = pio->cancel();
-    delete pio;
-    return status;
-}
-
-int shell::wait(shell::pipe_t pio)
-{
-    int status = pio->wait();
-    delete pio;
-    return status;
-}
-
-shell::pipe_t shell::spawn(const char *path, char **argv, pmode_t mode, size_t size, char **env)
-{
-    shell::pipe_t pio = new shell::pipeio;
-    if(pio->spawn(path, argv, mode, size, env)) {
-        delete pio;
-        return NULL;
-    }
-    return pio;
-}
-
 #ifdef _MSWINDOWS_
 
 static void pathfinder(const char *name, char *buf, size_t size)
@@ -1463,7 +1077,6 @@ const char *shell::getenv(const char *id, const char *value)
 int shell::system(const char *cmd, const char **envp)
 {
     char cmdspec[128];
-    DWORD code = -1;
     PROCESS_INFORMATION pi;
     char *ep = NULL;
     unsigned len = 0;
@@ -1790,6 +1403,13 @@ int shell::inkey(const char *prompt)
     return (char)getch();
 }
 
+int shell::cancel(shell::pid_t pid)
+{
+    if(!TerminateProcess(pid, 255))
+        return -1;
+    return 0;
+}
+
 char *shell::getline(const char *prompt, char *buffer, size_t size)
 {
     unsigned pos = 0;
