@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
 #include <ucommon/string.h>
 #include <stdarg.h>
@@ -278,7 +278,7 @@ string::string(long value)
 {
     str = create(20);
     str->retain();
-    snprintf(str->text, 20, "%ld", value);
+    snprintf(&str->text[0], 20, "%ld", value);
     str->len = strlen(str->text);
     str->fix();
 }
@@ -287,7 +287,7 @@ string::string(double value)
 {
     str = create(32);
     str->retain();
-    snprintf(str->text, 32, "%f", value);
+    snprintf(&str->text[0], 32, "%f", value);
     str->len = strlen(str->text);
     str->fix();
 }
@@ -411,22 +411,6 @@ int string::compare(const char *s) const
 #endif
 }
 
-const char *string::last(const char *clist) const
-{
-    if(!str)
-        return NULL;
-
-    return last(str->text, clist);
-}
-
-const char *string::first(const char *clist) const
-{
-    if(!str)
-        return NULL;
-
-    return first(str->text, clist);
-}
-
 const char *string::begin(void) const
 {
     if(!str)
@@ -573,6 +557,80 @@ void string::trim(const char *clist)
     str->fix();
 }
 
+unsigned string::replace(const char *substring, const char *cp, unsigned flags)
+{
+    const char *result = "";
+    unsigned count = 0;
+    size_t cpl = 0;
+
+    if(cp)
+        cpl = strlen(cp);
+
+    if(!str || !substring || str->len == 0)
+        return 0;
+
+    strsize_t offset = 0;
+
+    size_t tcl = strlen(substring);
+
+    while(result) {
+        const char *text = str->text + offset;
+#if defined(_MSWINDOWS_)
+        if((flags & 0x01) == INSENSITIVE)
+            result = strstr(text, substring);
+#elif  defined(HAVE_STRICMP)
+        if((flags & 0x01) == INSENSITIVE)
+            result = stristr(text, substring);
+#else
+        if((flags & 0x01) == INSENSITIVE)
+            result = strcasestr(text, substring);
+#endif
+        else
+            result = strstr(text, substring);
+
+        if(result) {
+            ++count;
+            offset = text - str->text;
+            cut(offset, tcl);
+            if(cpl) {
+                paste(offset, cp);
+                offset += cpl;
+            }
+        }
+    }
+    return count;
+}
+
+const char *string::search(const char *substring, unsigned instance, unsigned flags) const
+{
+    const char *result = "";
+
+    if(!str || !substring || str->len == 0)
+        return NULL;
+
+    const char *text = str->text;
+
+    if(!instance)
+        ++instance;
+    while(instance-- && result) {
+#if defined(_MSWINDOWS_)
+        if((flags & 0x01) == INSENSITIVE)
+            result = strstr(text, substring);
+#elif  defined(HAVE_STRICMP)
+        if((flags & 0x01) == INSENSITIVE)
+            result = stristr(text, substring);
+#else
+        if((flags & 0x01) == INSENSITIVE)
+            result = strcasestr(text, substring);
+#endif
+        else
+            result = strstr(text, substring);
+
+        if(result)
+            text = result + strlen(result);
+    }
+    return result;
+}
 
 const char *string::find(const char *clist, strsize_t offset) const
 {
@@ -828,6 +886,36 @@ void string::set(const char *s)
     str->set(s);
 }
 
+void string::paste(strsize_t offset, const char *cp, strsize_t size)
+{
+    if(!cp)
+        return;
+
+    if(!size)
+        size = strlen(cp);
+
+    if(!size)
+        return;
+
+    cow(size);
+
+    if(!str) {
+        string::set(str->text, ++size, cp);
+        str->len = --size;
+        str->fix();
+        return;
+    }
+
+    if(offset >= str->len)
+        string::set(str->text + str->len, size + 1, cp);
+    else {
+        memmove(str->text + offset + size, str->text + offset, str->len - offset);
+        memmove(str->text + offset, cp, size);
+    }
+    str->len += size;
+    str->fix();
+}
+
 void string::cut(strsize_t offset, strsize_t size)
 {
     if(!str || offset >= str->len)
@@ -847,6 +935,51 @@ void string::cut(strsize_t offset, strsize_t size)
     str->fix();
 }
 
+void string::paste(char *text, size_t max, size_t offset, const char *cp, size_t size)
+{
+    if(!cp || !text)
+        return;
+
+    if(!size)
+        size = strlen(cp);
+
+    if(!size)
+        return;
+
+    size_t len = strlen(text);
+    if(len <= max)
+        return;
+
+    if(len + size >= max)
+        size = max - len;
+
+    if(offset >= len)
+        string::set(text + len, size + 1, cp);
+    else {
+        memmove(text + offset + size, text + offset, len - offset);
+        memmove(text + offset, cp, size);
+    }
+}
+
+void string::cut(char *text, size_t offset, size_t size)
+{
+    if(!text)
+        return;
+
+    size_t len = strlen(text);
+    if(!len || offset >= len)
+        return;
+
+    if(offset + size >= len) {
+        text[offset] = 0;
+        return;
+    }
+
+    memmove(text + offset, text + offset + size, len - offset - size);
+    len -= size;
+    text[len] = 0;
+}
+
 bool string::resize(strsize_t size)
 {
     char fill = 0;
@@ -909,6 +1042,25 @@ void string::cow(strsize_t size)
     }
 }
 
+void string::add(char ch)
+{
+    char buf[2];
+
+    if(ch == 0)
+        return;
+
+    buf[0] = ch;
+    buf[1] = 0;
+
+    if(!str) {
+        set(buf);
+        return;
+    }
+
+    cow(1);
+    str->add(buf);
+}
+
 void string::add(const char *s)
 {
     if(!s || !*s)
@@ -919,6 +1071,7 @@ void string::add(const char *s)
         return;
     }
 
+    cow(strlen(s));
     str->add(s);
 }
 
@@ -1002,17 +1155,25 @@ string &string::operator--()
 string &string::operator+=(strsize_t offset)
 {
     if(str)
-        str->dec(offset);
+        str->inc(offset);
     return *this;
 }
 
 string &string::operator-=(strsize_t offset)
 {
     if(str)
-        str->inc(offset);
+        str->dec(offset);
     return *this;
 }
 
+bool string::operator*=(const char *substr)
+{
+    if(search(substr))
+        return true;
+
+    return false;
+}
+
 string &string::operator^=(const char *s)
 {
     release();
@@ -1021,6 +1182,19 @@ string &string::operator^=(const char *s)
 }
 
 string &string::operator=(const char *s)
+{
+    release();
+    set(s);
+    return *this;
+}
+
+string &string::operator|=(const char *s)
+{
+    set(s);
+    return *this;
+}
+
+string &string::operator&=(const char *s)
 {
     set(s);
     return *this;
@@ -1124,22 +1298,43 @@ string &string::operator&(const char *s)
     return *this;
 }
 
+string &string::operator|(const char *s)
+{
+    if(!s || !*s)
+        return *this;
+
+    if(!str) {
+        set(s);
+        return *this;
+    }
+
+    str->add(s);
+    return *this;
+}
+
 string &string::operator+(const char *s)
 {
     if(!s || !*s)
         return *this;
 
-    cow(strlen(s));
     add(s);
     return *this;
 }
 
+string string::operator^(const char *s)
+{
+    string tmp = s;
+    if(s && *s)
+        tmp += s;
+
+    return tmp;
+}
+
 string &string::operator+=(const char *s)
 {
     if(!s || !*s)
         return *this;
 
-    cow(strlen(s));
     add(s);
     return *this;
 }
@@ -1311,6 +1506,44 @@ char *string::dup(const char *cp)
     return mem;
 }
 
+char *string::left(const char *cp, size_t size)
+{
+    char *mem;
+
+    if(!cp)
+        return NULL;
+
+    if(!size)
+        size = strlen(cp);
+
+    mem = (char *)malloc(++size);
+    crit(mem != NULL, "string dup allocation error");
+    string::set(mem, size, cp);
+    return mem;
+}
+
+const char *string::pos(const char *cp, ssize_t offset)
+{
+    if(!cp)
+        return NULL;
+
+    size_t len = strlen(cp);
+    if(!len)
+        return cp;
+
+    if(offset >= 0) {
+        if((size_t)offset > len)
+            offset = len;
+        return cp + offset;
+    }
+
+    offset = -offset;
+    if((size_t)offset >= len)
+        return cp;
+
+    return cp + len - offset;
+}
+
 size_t string::count(const char *cp)
 {
     if(!cp)
@@ -1564,6 +1797,24 @@ char *string::rskip(char *str, const char *clist)
     return NULL;
 }
 
+size_t string::seek(char *str, const char *clist)
+{
+    size_t pos = 0;
+
+    if(!str)
+        return 0;
+
+    if(!clist)
+        return strlen(str);
+
+    while(str[pos]) {
+        if(strchr(clist, str[pos]))
+            return pos;
+        ++pos;
+    }
+    return pos;
+}
+
 char *string::find(char *str, const char *clist)
 {
     if(!str)
@@ -1596,47 +1847,6 @@ char *string::rfind(char *str, const char *clist)
     return NULL;
 }
 
-char *string::last(char *str, const char *clist)
-{
-    char *cp, *lp = NULL;
-
-    if(!str)
-        return NULL;
-
-    if(!clist)
-        return str + strlen(str) - 1;
-
-    while(clist && *clist) {
-        cp = strrchr(str, *(clist++));
-        if(cp && cp > lp)
-            lp = cp;
-    }
-
-    return lp;
-}
-
-char *string::first(char *str, const char *clist)
-{
-    char *cp, *fp;
-
-    if(!str)
-        return NULL;
-
-    if(!clist)
-        return str;
-
-    fp = str + strlen(str);
-    while(clist && *clist) {
-        cp = strchr(str, *(clist++));
-        if(cp && cp < fp)
-            fp = cp;
-    }
-
-    if(!*fp)
-        fp = NULL;
-    return fp;
-}
-
 bool string::case_equal(const char *s1, const char *s2)
 {
     if(!s1)
@@ -2122,72 +2332,3 @@ uint16_t string::crc16(uint8_t *binary, size_t size)
     return crc;
 }
 
-NAMESPACE_UCOMMON
-
-#ifdef  HAVE_STRCOLL
-
-bool sort_eq(char const *str1, char const *str2)
-{
-    return strcoll(str1, str2) == 0;
-}
-
-bool sort_ne(char const *str1, char const *str2)
-{
-    return strcoll(str1, str2) != 0;
-}
-
-bool sort_lt(char const *str1, char const *str2)
-{
-    return strcoll(str1, str2) < 0;
-}
-
-bool sort_gt(char const *str1, char const *str2)
-{
-    return strcoll(str1, str2) > 0;
-}
-
-bool sort_le(char const *str1, char const *str2)
-{
-    return strcoll(str1, str2) <= 0;
-}
-
-bool sort_ge(char const *str1, char const *str2)
-{
-    return strcoll(str1, str2) >= 0;
-}
-
-#else
-
-bool sort_eq(char const *str1, char const *str2)
-{
-    return strcmp(str1, str2) == 0;
-}
-
-bool sort_ne(char const *str1, char const *str2)
-{
-    return strcmp(str1, str2) != 0;
-}
-
-bool sort_lt(char const *str1, char const *str2)
-{
-    return strcmp(str1, str2) < 0;
-}
-
-bool sort_gt(char const *str1, char const *str2)
-{
-    return strcmp(str1, str2) > 0;
-}
-
-bool sort_le(char const *str1, char const *str2)
-{
-    return strcmp(str1, str2) <= 0;
-}
-
-bool sort_ge(char const *str1, char const *str2)
-{
-    return strcmp(str1, str2) >= 0;
-}
-
-#endif
-
-END_NAMESPACE
