@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
 #include <ucommon/object.h>
 #include <ucommon/thread.h>
@@ -199,7 +199,7 @@ void Conditional::gettimeout(timeout_t msec, struct timespec *ts)
 #endif
     ts->tv_sec += msec / 1000;
     ts->tv_nsec += (msec % 1000) * 1000000l;
-    while(ts->tv_nsec > 1000000000l) {
+    while(ts->tv_nsec >= 1000000000l) {
         ++ts->tv_sec;
         ts->tv_nsec -= 1000000000l;
     }
@@ -579,7 +579,7 @@ void ConditionalAccess::waitBroadcast(void)
 
 ConditionalAccess::ConditionalAccess() : Conditional()
 {
-    pending = waiting = pending = 0;
+    pending = waiting = sharing = 0;
 }
 
 ConditionalAccess::~ConditionalAccess()
