@@ -15,8 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
+#include <ucommon/protocols.h>
+#include <ucommon/object.h>
 #include <ucommon/vector.h>
 #include <ucommon/thread.h>
 #include <string.h>
