@@ -15,16 +15,14 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with GNU uCommon C++.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <ucommon-config.h>
 #include <ucommon/export.h>
+#include <ucommon/protocols.h>
 #include <ucommon/string.h>
 #include <ucommon/xml.h>
 #include <ctype.h>
 
 using namespace UCOMMON_NAMESPACE;
-#if !defined(_MSC_VER)
-using namespace std;
-#endif
 
 static bool isElement(char c)
 {
