@@ -43,6 +43,10 @@
 #include <ucommon/fsys.h>
 #endif
 
+#ifndef _UCOMMON_SHELL_H_
+#include <ucommon/shell.h>
+#endif
+
 NAMESPACE_UCOMMON
 
 /**
@@ -55,6 +59,7 @@ class fbuf : public BufferProtocol, private fsys
 {
 private:
     offset_t    inpos, outpos;
+    shell::pid_t pid;
 
 protected:
     size_t _push(const char *address, size_t size);
@@ -85,6 +90,15 @@ public:
      */
     fbuf(const char *path, fsys::access_t access, unsigned permissions, size_t size);
 
+    /**
+     * Construct a pipe buffer from an existing file.
+     * @param path of existing process.
+     * @param access mode of pipe.
+     * @param argv to pass.
+     * @param envp to use.
+     */
+    fbuf(const char *path, fsys::access_t access, char **argv, size_t size, char **envp = NULL);
+
     /**
      * Construct a file buffer that opens an existing file.
      * @param path of existing file to open.
@@ -112,10 +126,27 @@ public:
     void open(const char *path, fsys::access_t access = fsys::ACCESS_RDWR, size_t size = 512);
 
     /**
-     * Close the file, flush buffers.
+     * Construct a pipe buffer from an existing file.
+     * @param path of existing process.
+     * @param access mode of pipe.
+     * @param argv to pass.
+     * @param envp to use.
+     */
+    void open(const char *path, fsys::access_t access, char **argv, size_t size = 512, char **envp = NULL);
+
+    /**
+     * Close the file, flush buffers.  Capture exit code in error for pipe.
      */
     void close(void);
 
+    /**
+     * Force terminate child and close.
+     */
+    void terminate(void);
+
+    inline void cancel(void)
+        {terminate();}
+
     /**
      * Seek specific offset in open file and reset I/O buffers.  If the
      * file is opened for both read and write, both the read and write
