@@ -177,6 +177,15 @@ public:
      */
     bool copy(void *data, timeout_t timeout);
 
+    /**
+     * Peek at pending data in buffer.  This returns a pointer to
+     * objects relative to the head.  In effect it is the same as
+     * get() for item = 0.
+     * @param item to examine in buffer.
+     * @return pointer to item or NULL if invalid item number.
+     */
+    void *peek(unsigned item);
+
     /**
      * Test if there is data waiting in the buffer.
      * @return true if buffer has data.
@@ -253,6 +262,13 @@ public:
      */
     bool post(ObjectProtocol *object, timeout_t timeout = 0);
 
+    /**
+     * Examine pending existing object in queue.  Does not remove it.
+     * @param number of elements back.
+     * @return object in queue or NULL if invalid value.
+     */
+    ObjectProtocol *peek(unsigned offset = 0);
+
     /**
      * Get and remove last object posted to the queue.  This can wait for
      * a specified timeout of the queue is empty.  The object is still
@@ -321,6 +337,9 @@ public:
      */
     static size_t count(queue& queue)
         {return queue.getCount();};
+
+    inline ObjectProtocol *operator[](unsigned pos)
+        {return peek(pos);};
 };
 
 /**
@@ -392,6 +411,13 @@ public:
      */
     ObjectProtocol *pull(timeout_t timeout = 0);
 
+    /**
+     * Examine an existing object on the stack.
+     * @param offset to stack entry.
+     * @return object examined.
+     */
+    ObjectProtocol *peek(unsigned offset = 0);
+
     /**
      * Get number of object points currently in the stack.
      * @return number of objects in stack.
@@ -433,6 +459,9 @@ public:
      */
     static inline size_t count(stack& stack)
         {return stack.getCount();};
+
+    inline ObjectProtocol *operator[](unsigned pos)
+        {return peek(pos);};
 };
 
 /**
@@ -539,6 +568,25 @@ public:
      */
     inline bool get(T *object, timeout_t timeout)
         {return copy(object, timeout);};
+
+    /**
+     * Examine past item in the buffer.  This is a typecast of the peek
+     * operation.
+     * @param item in buffer.
+     * @return item pointer if valid or NULL.
+     */
+    inline T *peek(unsigned item)
+        {return static_cast<T *>(Buffer::peek(item));};
+
+    /**
+     * Examine past item in the buffer.  This is a typecast of the peek
+     * operation.
+     * @param item in buffer.
+     * @return item pointer if valid or NULL.
+     */
+    inline T *operator[](unsigned item)
+        {return static_cast<T *>(queue::peek(item));};
+
 };
 
 /**
@@ -589,6 +637,25 @@ public:
      */
     inline T *pull(timeout_t timeout = 0)
         {return static_cast<T *>(stack::pull(timeout));};
+
+    /**
+     * Examine past item in the stack.  This is a typecast of the peek
+     * operation.
+     * @param offset in stack.
+     * @return item pointer if valid or NULL.
+     */
+    inline T *peek(unsigned offset = 0)
+        {return static_cast<T *>(stack::peek(offset));};
+
+    /**
+     * Examine past item in the stack.  This is a typecast of the peek
+     * operation.
+     * @param offset in stack.
+     * @return item pointer if valid or NULL.
+     */
+    inline T *operator[](unsigned offset)
+        {return static_cast<T *>(stack::peek(offset));};
+
 };
 
 /**
@@ -649,6 +716,25 @@ public:
      */
     inline T *lifo(timeout_t timeout = 0)
         {return static_cast<T *>(queue::lifo(timeout));};
+
+    /**
+     * Examine past item in the queue.  This is a typecast of the peek
+     * operation.
+     * @param offset in queue.
+     * @return item pointer if valid or NULL.
+     */
+    inline T *peek(unsigned offset = 0)
+        {return static_cast<T *>(queue::peek(offset));};
+
+    /**
+     * Examine past item in the queue.  This is a typecast of the peek
+     * operation.
+     * @param offset in queue.
+     * @return item pointer if valid or NULL.
+     */
+    inline T *operator[](unsigned offset)
+        {return static_cast<T *>(queue::peek(offset));};
+
 };
 
 /**
