@@ -243,6 +243,13 @@ public:
      */
     void operator=(const fsys& descriptor);
 
+    /**
+     * Replace current file descriptor with an external descriptor.  This
+     * does not create a duplicate.  The external descriptor object is
+     * marked as invalid.
+     */
+    void operator*=(fd_t& descriptor);
+
     /**
      * Assing file descriptor from system descriptor.
      * @param descriptor to dup from.
@@ -593,6 +600,31 @@ public:
      */
     static void release(fd_t descriptor);
 
+    /**
+     * Create pipe.  These are created inheritable by default.
+     * @param input descriptor.
+     * @param output descriptor.
+     * @param size of buffer if supported.
+     * @return 0 or error code.
+     */
+    static int pipe(fd_t& input, fd_t& output, size_t size = 0);
+
+    /**
+     * Changle inheritable handle.  On windows this is done by creating a
+     * duplicate handle and then closing the original.  Elsewhere this
+     * is done simply by setting flags.
+     * @param descriptor to modify.
+     * @param enable child process inheritence.
+     * @return 0 on success, error on failure.
+     */
+    static int inherit(fd_t& descriptor, bool enable);
+
+    /**
+     * Create inheritable /dev/null handle.
+     * @return null device handle.
+     */
+    static fd_t nullfile(void);
+
     /**
      * create a file descriptor or directory directly.
      * @param object to assign descriptor for created file.
@@ -618,6 +650,8 @@ public:
      */
     static int exec(const char *path, char **argv);
 
+    static int exec(const char *path, char **argv, char **envp);
+
     /**
      * Load a plugin into memory.
      * @param module for management.
@@ -717,6 +751,9 @@ public:
     inline bool operator !()
         {return fp == NULL;}
 
+    inline operator FILE *()
+        {return fp;}
+
     /**
      * Open file path.  If a file is already opened, it is closed.
      * @param path of file to open.
