@@ -94,6 +94,42 @@ protected:
      */
 
 public:
+    enum {
+        SENSITIVE = 0x00,
+        INSENSITIVE = 0x01
+    };
+
+    class __EXPORT regex
+    {
+    private:
+        void *object;
+        void *results;
+        size_t count;
+
+    public:
+        regex(const char *pattern, size_t slots = 1);
+        regex(size_t slots = 1);
+        ~regex();
+
+        size_t offset(unsigned member);
+        size_t size(unsigned member);
+
+        inline size_t members(void)
+            {return count;};
+
+        bool match(const char *text, unsigned flags = 0);
+
+        regex& operator=(const char *string);
+
+        bool operator*=(const char *string);
+
+        operator bool()
+            {return object != NULL;}
+
+        bool operator!()
+            {return object == NULL;}
+    };
+
     class __EXPORT cstring : public CountedObject
     {
     public:
@@ -192,6 +228,7 @@ protected:
      */
     cstring *create(strsize_t size, char fill = 0) const;
 
+public:
     /**
      * Compare the values of two string.  This is a virtual so that it
      * can be overridden for example if we want to create strings which
@@ -204,6 +241,7 @@ protected:
     inline int collate(const char *string) const
         {return compare(string);};
 
+protected:
     /**
     * Test if two string values are equal.
     * @param string to compare with.
@@ -433,12 +471,26 @@ public:
      */
     void trim(const char *list);
 
+    /**
+     * Trim lead characters from text.
+     * @param count of characters to remove.
+     */
+    inline void trim(strsize_t count = 1)
+        {operator+=(count);};
+
     /**
      * Chop trailing characters from the string.
      * @param list of characters to remove.
      */
     void chop(const char *list);
 
+    /**
+     * Chop trailing characters from text.
+     * @param count of characters to remove.
+     */
+    inline void chop(strsize_t count = 1)
+        {operator-=(count);};
+
     /**
      * Strip lead and trailing characters from the string.
      * @param list of characters to remove.
@@ -459,6 +511,14 @@ public:
      */
     void cut(strsize_t offset, strsize_t size = 0);
 
+    /**
+     * Insert (paste) text into string.
+     * @param offset to start paste.
+     * @param text to paste.
+     * @param size of text to paste.
+     */
+    void paste(strsize_t offset, const char *text, strsize_t size = 0);
+
     /**
      * Clear a field of a filled string with filler.
      * @param offset to start of field to clear.
@@ -518,20 +578,6 @@ public:
      */
     char at(int position) const;
 
-    /**
-     * Find last occurrence of a character in the string.
-     * @param list of characters to search for.
-     * @return pointer to last occurrence from list or NULL.
-     */
-    const char *last(const char *list) const;
-
-    /**
-     * Find first occurrence of a character in the string.
-     * @param list of characters to search for.
-     * @return pointer to first occurrence from list or NULL.
-     */
-    const char *first(const char *list) const;
-
     /**
      * Get pointer to first character in string for iteration.
      * @return first character pointer or NULL if empty.
@@ -561,6 +607,19 @@ public:
      */
     const char *rskip(const char *list, strsize_t offset = npos) const;
 
+    /**
+     * Search for a substring in the string.
+     * @param substring to search for.
+     * @param flags for case insensitive search.
+     */
+    const char *search(const char *string, unsigned instance = 0, unsigned flags = 0) const;
+
+    const char *search(regex& expr, unsigned instance = 0, unsigned flags = 0) const;
+
+    unsigned replace(const char *string, const char *text = NULL, unsigned flags = 0);
+
+    unsigned replace(regex& expr, const char *text = NULL, unsigned flags = 0);
+
     /**
      * Find a character in the string.
      * @param list of characters to search for.
@@ -660,6 +719,31 @@ public:
      */
     string operator()(int offset, strsize_t size) const;
 
+    /**
+     * Convenience method for left of string.
+     * @param size of substring to gather.
+     * @return string object holding substring.
+     */
+    inline string left(strsize_t size) const
+        {return operator()(0, size);}
+
+    /**
+     * Convenience method for right of string.
+     * @param offset of substring from right.
+     * @return string object holding substring.
+     */
+    inline string right(strsize_t offset) const
+        {return operator()(-((int)offset), 0);}
+
+    /**
+     * Convenience method for substring extraction.
+     * @param offset into string.
+     * @param size of string to return.
+     * @return string object holding substring.
+     */
+    inline string copy(strsize_t offset, strsize_t size) const
+        {return operator()((int)offset, size);}
+
     /**
      * Reference a string in the object by relative offset.  Positive
      * offsets are from the start of the string, negative from the
@@ -695,6 +779,14 @@ public:
      */
     string& operator^=(const string& object);
 
+    /**
+     * Concatenate text to an existing string object.  This will use the
+     * old behavior when +/= updated.
+     */
+    string& operator|=(const char *text);
+
+    string& operator&=(const char *text);
+
     /**
      * Concatenate text to an existing string object.
      * @param text to add.
@@ -711,11 +803,27 @@ public:
 
     /**
      * Concatenate null terminated text to our object.  This creates a new
-     * copy-on-write instance to hold the concatenated string.
+     * copy-on-write instance to hold the concatenated string.  This will
+     * eventually become same as new ^ operator in 6.0.
      * @param text to concatenate.
      */
     string& operator+(const char *text);
 
+    /**
+     * Concatenate null terminated text to our object.  This creates a new
+     * copy-on-write instance to hold the concatenated string.  This will
+     * eventually replace '+' when + creates a new string instance instead.
+     * @param text to concatenate.
+     */
+    string& operator|(const char *text);
+
+    /**
+     * Transitional concat operator until abi can be changed.
+     * @param text to concatenate.
+     */
+    string operator^(const char *text);
+
+
     /**
      * Concatenate null terminated text to our object.  This directly
      * appends the text to the string buffer and does not resize the
@@ -732,6 +840,10 @@ public:
      */
     string& operator=(const string& object);
 
+    bool operator*=(const char *substring);
+
+    bool operator*=(regex& expr);
+
     /**
      * Assign text to our existing buffer.  This performs a set method.
      * @param text to assign from.
@@ -760,6 +872,12 @@ public:
      */
     string& operator-=(strsize_t number);
 
+    /**
+     * Delete a specified number of characters from start of string.
+     * @param number of characters to delete.
+     */
+    string& operator*=(strsize_t number);
+
     /**
      * Compare our object with null terminated text.
      * @param text to compare with.
@@ -802,6 +920,12 @@ public:
      */
     bool operator>=(const char *text) const;
 
+    inline string& operator<<(const char *text)
+        {add(text); return *this;}
+
+    inline string& operator<<(char code)
+        {add(code); return *this;}
+
     /**
      * Parse short integer value from a string.
      * @param value to store.
@@ -1127,6 +1251,14 @@ public:
      */
     static char *find(char *text, const char *list);
 
+    /**
+     * Offset until next occurrence of character in a text or length.
+     * @param text buffer to examine.
+     * @param list of characters to search for.
+     * @return offset to next occurrence or length of string.
+     */
+    static size_t seek(char *text, const char *list);
+
     /**
      * Find the last occurrence of a character in a text buffer.
      * @param text buffer to examine.
@@ -1135,22 +1267,6 @@ public:
      */
     static char *rfind(char *text, const char *list);
 
-    /**
-     * Get pointer to first character past character requested.
-     * @param text buffer to examine.
-     * @param list of characters.
-     * @return first character pointer past list.
-     */
-    static char *first(char *text, const char *list);
-
-    /**
-     * Get pointer to last character before character requested.
-     * @param text buffer to examine.
-     * @param list of characters.
-     * @return last character pointer past list.
-     */
-    static char *last(char *text, const char *list);
-
     /**
      * Duplicate null terminated text into the heap.
      * @param text to duplicate.
@@ -1158,6 +1274,32 @@ public:
      */
     static char *dup(const char *text);
 
+    /**
+     * Duplicate null terminated text of specific size to heap.
+     * @param text to duplicate.
+     * @param size of text, maximum space allocated.
+     * @return duplicate copy of text allocated on heap.
+     */
+    static char *left(const char *text, size_t size);
+
+    /**
+     * Compute position in string.
+     * @param text of string.
+     * @param offset from start, negative values from end.
+     * @return pointer to string position.
+     */
+    static const char *pos(const char *text, ssize_t offset);
+
+    inline static char *right(const char *text, size_t size)
+        {return dup(pos(text, -(signed)size));}
+
+    inline static char *copy(const char *text, size_t offset, size_t len)
+        {return left(pos(text, offset), len);}
+
+    static void cut(char *text, size_t offset, size_t len);
+
+    static void paste(char *text, size_t max, size_t offset, const char *data, size_t len = 0);
+
     /**
      * A thread-safe token parsing routine for strings objects.  This
      * is related to strtok, but with safety checks for NULL values and a
@@ -1240,21 +1382,36 @@ public:
      * @return count of characters.
      */
     inline static size_t count(string& object)
-        {return object.count();};
+        {return object.count();}
+
+    inline static string copy(string& object, strsize_t offset, strsize_t size)
+        {return object.copy(offset, size);}
+
+    inline static string left(string& object, strsize_t size)
+        {return object.left(size);}
+
+    inline static string right(string& object, strsize_t offset)
+        {return object.right(offset);}
+
+    inline static void cut(string& object, strsize_t offset, strsize_t size)
+        {object.cut(offset, size);}
+
+    inline static void paste(string& object, strsize_t offset, const char *text, strsize_t size = 0)
+        {object.paste(offset, text, size);}
 
     /**
      * Convert string object to upper case.
      * @param object to modify.
      */
     inline static void upper(string& object)
-        {object.upper();};
+        {object.upper();}
 
     /**
      * Convert string object to lower case.
      * @param object to modify.
      */
     inline static void lower(string& object)
-        {object.lower();};
+        {object.lower();}
 
     /**
      * Unquote a quoted string.  Removes lead and trailing quote marks.
@@ -1263,7 +1420,7 @@ public:
      * @return true if string was quoted.
      */
     inline static bool unquote(string& object, const char *quote)
-        {return object.unquote(quote);};
+        {return object.unquote(quote);}
 
     /**
      * Trim lead characters from the string.
@@ -1271,7 +1428,15 @@ public:
      * @param list of characters to remove.
      */
     inline static void trim(string& object, const char *list)
-        {object.trim(list);};
+        {object.trim(list);}
+
+    /**
+     * Trim first character(s) from the string.
+     * @param object to chop.
+     * @param count of characters to remove.
+     */
+    inline static void trim(string& object, strsize_t count = 1)
+        {object.trim(count);}
 
     /**
      * Chop trailing characters from the string.
@@ -1279,7 +1444,15 @@ public:
      * @param list of characters to remove.
      */
     inline static void chop(string& object, const char *list)
-        {object.trim(list);};
+        {object.chop(list);}
+
+    /**
+     * Chop last character(s) from the string.
+     * @param object to chop.
+     * @param count of characters to remove.
+     */
+    inline static void chop(string& object, strsize_t count = 1)
+        {object.chop(count);}
 
     /**
      * Strip lead and trailing characters from the string.
@@ -1287,7 +1460,7 @@ public:
      * @param list of characters to remove.
      */
     inline static void strip(string& object, const char *list)
-        {object.trim(list);};
+        {object.strip(list);}
 
     /**
      * Find a character in the string.
@@ -1296,7 +1469,22 @@ public:
      * @return pointer to first occurrence of character.
      */
     inline static const char *find(string& object, const char *list)
-        {return object.find(list);};
+        {return object.find(list);}
+
+    inline static const char *search(string& object, const char *substring, unsigned instance = 0, unsigned flags = 0)
+        {return object.search(substring, instance, flags);}
+
+    inline static const char *search(string& object, regex& expr, unsigned instance = 0, unsigned flags = 0)
+        {return object.search(expr, instance, flags);}
+
+    inline static unsigned replace(string& object, regex& expr, const char *text, unsigned flags = 0)
+        {return object.replace(expr, text, flags);}
+
+    inline static unsigned replace(string& object, const char *match, const char *text, bool flag = false)
+        {return object.replace(match, text, flag);}
+
+    inline static bool search(regex& expr, const char *text, unsigned flags = 0)
+        {return expr.match(text, flags);}
 
     /**
      * Find last character in the string.
@@ -1305,25 +1493,7 @@ public:
      * @return pointer to last occurrence of character.
      */
     inline static const char *rfind(string& object, const char *list)
-        {return object.rfind(list);};
-
-    /**
-     * Get pointer to first character past character requested.
-     * @param object to examine.
-     * @param list of characters.
-     * @return first character pointer past list.
-     */
-    inline static const char *first(string& object, const char *list)
-        {return object.first(list);};
-
-    /**
-     * Get pointer to last character past character requested.
-     * @param object to examine.
-     * @param list of characters.
-     * @return last character pointer before list.
-     */
-    inline static const char *last(string& object, const char *list)
-        {return object.last(list);};
+        {return object.rfind(list);}
 
     /**
      * Convert string to a double value.
@@ -1332,7 +1502,7 @@ public:
      * @return double value of object.
      */
     inline static double tod(string& object, char **pointer = NULL)
-        {return strtod(mem(object), pointer);};
+        {return strtod(mem(object), pointer);}
 
     /**
      * Convert string to a long value.
@@ -1341,7 +1511,7 @@ public:
      * @return long value of object.
      */
     inline static long tol(string& object, char **pointer = NULL)
-        {return strtol(mem(object), pointer, 0);};
+        {return strtol(mem(object), pointer, 0);}
 
     /**
      * Convert text to a double value.
@@ -1350,7 +1520,7 @@ public:
      * @return double value of object.
      */
     inline static double tod(const char *text, char **pointer = NULL)
-        {return strtod(text, pointer);};
+        {return strtod(text, pointer);}
 
     /**
      * Convert text to a long value.
@@ -1359,7 +1529,7 @@ public:
      * @return long value of object.
      */
     inline static long tol(const char *text, char **pointer = NULL)
-        {return strtol(text, pointer, 0);};
+        {return strtol(text, pointer, 0);}
 
     /**
      * Standard radix 64 encoding.
@@ -1585,6 +1755,8 @@ public:
  */
 typedef string string_t;
 
+typedef string::regex stringex_t;
+
 /**
  * A convenience type when mixing std::string in old compilers that are bad
  * with namespaces...
@@ -1683,16 +1855,31 @@ inline bool ne(char const *s1, char const *s2, size_t size)
     {return !String::equal(s1, s2, size);}
 
 /**
- * Compare two string objects if equal.
+ * Compare two string objects if equal.  The left string is an object,
+ * the right may be an object or converted to a const string.  The
+ * compare virtual method of the left object is used, so we can do
+ * things like collation order or derived class specific sorting.
  * @param s1 string to compare.
  * @param s2 string to compare.
  * @return true if equal.
  */
-inline bool eq(String &s1, String &s2)
-    {return String::equal(s1.c_str(), s2.c_str());}
+inline bool eq(String &s1, const char *s2)
+    {return s1.compare(s2) == 0;}
 
 inline bool ne(String &s1, String &s2)
-    {return !String::equal(s1.c_str(), s2.c_str());}
+    {return s1.compare(s2) != 0;}
+
+inline bool lt(String &s1, const char *s2)
+    {return s1.compare(s2) < 0;}
+
+inline bool gt(String &s1, const char *s2)
+    {return s1.compare(s2) > 0;}
+
+inline bool le(String &s1, const char *s2)
+    {return s1.compare(s2) <= 0;}
+
+inline bool ge(String &s1, const char *s2)
+    {return s1.compare(s2) >= 0;}
 
 /**
  * Compare two null terminated strings if equal ignoring case.  This is
@@ -1725,16 +1912,6 @@ inline bool case_eq(char const *s1, char const *s2, size_t size)
 inline bool ieq(char const *s1, char const *s2, size_t size)
     {return String::case_equal(s1, s2, size);}
 
-/**
- * Compare two string objects if equal ignoring case.  This is
- * related to stricmp or gcc strcasecmp.
- * @param s1 string to compare.
- * @param s2 string to compare.
- * @return true if equal.
- */
-inline bool ieq(String &s1, String &s2)
-    {return String::case_equal(s1.c_str(), s2.c_str());}
-
 inline String str(const char *string)
     {return (String)string;}
 
@@ -1758,13 +1935,6 @@ inline String str(double value)
 
 String str(CharacterProtocol& cp, strsize_t size);
 
-bool sort_eq(char const *str1, char const *str2);
-bool sort_ne(char const *str1, char const *str2);
-bool sort_gt(char const *str1, char const *str2);
-bool sort_lt(char const *str1, char const *str2);
-bool sort_ge(char const *str1, char const *str2);
-bool sort_le(char const *str1, char const *str2);
-
 template<>
 inline void swap<string_t>(string_t& s1, string_t& s2)
     {String::swap(s1, s2);}
@@ -1802,9 +1972,11 @@ public:
     inline const char *operator*() const
         {return data;}
 
-    inline char operator[](size_t size) const
+    inline char& operator[](int size)
         {return data[size];}
 
+    inline char *operator+(size_t size)
+        {return data + size;}
 };
 
 
